
import random
import model


class Bagging(model.Committee):
    """
    The Bagging algorithm is implemented as extension of a committee of models.
    Specifically, the committee is extended by a simple fitting method.
    
    As the evaluation (and error computation) is dependent on the specific
    problem, use the :class:`Committee` mixins like so:
    
    >>> class foo(Bagging, Committee.Classification): pass
    
    The models have to be assigned to the instance before training. For this,
    the method :func:`Committee.addModel` can be used.
    
    """
    def __init__(self):
        model.Committee.__init__(self)
    
    def fit(self, data):
        """Fit the assigned models to sets, :term:`bootstrapped` from *data*."""
        for m in self.models:
            # bootstrap sample from data
            samples = [random.choice(data) for i in range(len(data))]
            
            # train m on bootstrap samples
            m.fit(samples)
        return self
