
from numpy.random import uniform


def rejectionSamplerN(qSampler, qDist, pDist, N, M):
    """Draw samples from a distribution, given its probability density function.
    
    This sampler can be used, if the probability density function of
    the target distribution is known, but there's no direct sampling
    approach (e.g. the inverse cdf is not known).
    
    :param qSampler:    Sampling function of the proposal distribution.
    :type qSampler:     a
    :param qDist:       Probability density function of the proposal
                        distribution.
    :type qDist:        (Num b) => a -> b
    :param pDist:       Probability density function of the target distribution.
    :type pDist:        (Num b) => a -> b
    :param N:           Number of samples.
    :type N:            int
    :param M:           Scale parameter.
    :type M:            float
    :returns:           N samples, distributed according to the probability
                        density of the target distribution *pDist*.
    
    """
    g = rejectionSampler(qSampler, qDist, pDist, M)
    return [g.next() for i in range(N)]


def rejectionSampler(qSampler, qDist, pDist, M):
    """Draw samples from a distribution, given its probability density function.
    
    This sampler can be used, if the probability density function of
    the target distribution is known, but there's no direct sampling
    approach (e.g. the inverse cdf is not known).
    
    :param qSampler:    Sampling function of the proposal distribution.
    :type qSampler:     a
    :param qDist:       Probability density function of the proposal
                        distribution.
    :type qDist:        (Num b) => a -> b
    :param pDist:       Probability density function of the target distribution.
    :type pDist:        (Num b) => a -> b
    :param M:           Scale parameter.
    :type M:            float
    :returns:           Generator for samples, distributed according to the
                        probability density of the target distribution *pDist*.
    
    """
    while True:
        y = qSampler()
        u = uniform(); # uniform in [0,1]
        if u < pDist(y) / (M * qDist(y)):
            yield y

