
import math
import random

######### AVERAGE #########

# mean :: [num] -> num
# mean := genMean(lst, 1.0)
mean = lambda lst: sum(lst) / float(len(lst))
"""Return the arithmetic mean of all values of the given list."""


gMean = lambda lst: reduce(lambda x,y: x*y, lst)**(1.0/len(lst))
"""Return the geometric mean of the given values."""


hMean = lambda lst: mean(map(lambda x: 1.0/x, lst))
"""Return the harmonic mean of the given values."""


genMean = lambda lst,p: mean(map(lambda x: x**p, lst))**(1.0/p)
"""Return the generalized mean of the given values."""


# qMean := genMean(lst, 2.0)
qMean = lambda lst: math.sqrt(mean(map(lambda x: x**2, lst)))
"""Return the quadratic mean of the given values."""


rms = qMean
"""Return the root mean square of the given values."""


# cMean := genMean(lst, 3.0)
cMean = lambda lst: mean(map(lambda x: x**3, lst))**(1.0/3.0)
"""Return the cubic mean of the given values."""


# median :: [num] -> num
# lst :: [num]
def median(lst):
    """Return the median value of the given list."""
    s = sorted(lst)
    N = len(lst)
    if N % 2 == 1:
        return s[N / 2]
    else:
        return (s[N/2] + s[N/2-1])/2.0


midrange = lambda lst: (max(x) + min(x)) / 2.0
"""Return the midrange of the given values."""





######### MOMENTS #########

# var :: [num] -> num
_varM = lambda lst, m: sum(map(lambda i: (i - m)**2, lst)) / float(len(lst))
var = lambda lst: _varM(lst, mean(lst))
"""Return the variance of the given list."""


######### ERROR FUNCTION ######### 

rss = lambda lst: sum(map(lambda (x,y): (x-y)**2, lst))
"""Return the residual sum of squares.

The list items are expected to be of the form (f(x),y).
"""





######### RELATIVE FREQUENCY ######### 

def linsplit(data, binLen, key=None):
    """Split the sorted data into lists of equal length.
    
    >>> linsplit([6,3,4,6,8,85,6,5,4], 3)
    [[3,4,4],[5,6,6],[6,8,85]]
    
    :param data:        Input list
    :type data:         [a]
    :param key:         Sorting key. Default value is None.
    :returns:           [[a]]
    
    """
    d = sorted(data, key=key)
    return [d[i:i+binLen] for i in range(0,len(d),binLen)]



def binning(data, numBins, hist=False):
    """Divide a list into multiple bins of equal intervalls.
    
    For the type a there must be a float representation and it must be ordered.
    Specifically, the type must support the functions *min*, *max* and *float*.
    
    :param data:        Input list.
    :type data:         [a]
    :param numBins:     Number of bins.
    :type numBins:      int
    :param hist:        Return the number of samples per bin instead.
    :type hist:         bool
    :returns:           A list of bins, containing the data elements.
    
    >>> binning([1,2,3,4], 2, hist=False)
    [[1,2],[3,4]]
    
    """
    lo = min(data)
    hi = max(data)
    bins = [[] for i in range(numBins)]
    
    for x in data:
        # other indexing function ... not sure, if this one's correct
        #idx = min(numBins-1, int( math.floor((float(x)-lo) * numBins / (hi - lo)) ))
        idx = min(numBins-1, int( math.floor((x-lo) * numBins / (hi - lo)) ))
        bins[idx].append( x )
    
    if hist is True:
        bins = map(lambda i: len(i), bins)
    return bins


def binRange(data, numBins):
    """Return the intervall boundaries used by *binning*.
    
    The returned intervalls describe the lower and upper limit, whereas
    the lower limit is inclusive and the upper limit is inclusive only for the
    last intervall.
    
    :param data:        Input list.
    :type data:         a
    :param numBins:     Number of bins.
    :type numBins:      int
    :returns:           List of intervalls.
    
    >>> binRange([1,2,3,4], 2, hist=False)
    [(1.0,2.5),(2.5,4.0)]
    
    """
    dmin = min(data)
    s = max(data) - dmin
    range_ = []
    for i in range(numBins):
        lo = float(i) * s / numBins + dmin
        hi = float(i+1)* s / numBins + dmin
        range_.append( (lo,hi) )
    return range_


def histogram(data):
    """Count the number of occurrences of tokens in a list.
    
    :param data:        List of tokens.
    :returns:           Dict with token as key and relative frequency as value
    
    """
    hist = {}
    for a in data: hist[a] = 0.0
    for a in data:
        hist[a] += 1
    return hist


def majorityVote(data, tieBreaking=lambda l,h: random.choice(l)):
    """Return the token with the highest number of occurrences.

    The tie-breaking rule is applied, if there are several tokens with
    identical number of occurrences. The *tieBreaking* function takes
    two arguments: The tokens with maximal votes and the token
    histogram. It has to select a single token as a winner.
    The default function is a random choice.
    
    :param data:        Stream of input tokens.
    :type data:         a
    :param tieBreaking: Function to determine the vote winner, in the case of a
                        tie.
    :type tieBreaking:  [a] -> {a -> Int}  ->  a
    :returns:           Token of the data stream with the highest number of
                        occurrences.
    
    >>> majorityVote([1,2,1], tieBreaking=lambda l,h: max(l))
    1

    >>> majorityVote([1,2,1,2,3], tieBreaking=lambda l,h: max(l))
    2
    
    """
    hist = histogram(data)
    maxKey = max(hist, key=lambda k: hist[k])
    maxCand = filter(lambda k: hist[k] == hist[maxKey], hist)
    if len(maxCand) > 1:
        return tieBreaking(maxCand, hist)
    else:
        return maxKey

# draw N from lst without replacement           random.sample(lst, N)
# draw N from lst with replacement              [random.choice(lst) for i in range(N)]
# cartesian product                             itertools.product(lst1, lst2)
#  all combinations of two lists
